# -*- coding: utf-8 -*-
"""
Created on Mon Mar 18 2019

@author: Hannah Janout
"""
from PIL import Image, ImageChops
from PySpot import Ui_MainWindow
from matplotlib import pyplot as plt
import itertools as it
from matplotlib import cm
from mpl_toolkits.mplot3d import Axes3D
from joblib import Parallel, delayed
import multiprocessing
from multiprocessing import Pool
import os
from PyQt5 import QtCore, QtGui, QtWidgets
import qimage2ndarray as qi2np
import numpy as np
import cv2 as cv
import sys
import io
import ROI
import csv
import importlib
import math
import threading
import tifffile as tf

import read_spe2
import GraphicsScene
import ScaleMode
#import volview
from StackDataGUI import Ui_Dialog


class PySpot(Ui_MainWindow):

    def __init__(self, dialog):
        Ui_MainWindow.__init__(self)
        self.setupUi(dialog)
        self.QT_AUTO_SCREEN_SCALE_FACTOR = True
        self.scene = GraphicsScene.GraphicsSceneV()
        self.ratio = 1
        self.sceneScale = GraphicsScene.GraphicsSceneScale()
        self.gvScalebarTop.setScene(self.sceneScale)
        self.num_cores = multiprocessing.cpu_count()
        self.useThresh = False
        self.data = None
        # set Background
        self.txtBackground.setText("100")
        # buttons
        self._setButtonAction_()

        # signals
        self.scene.c.createROI.connect(self._refreshRoiAndStat_)
        self.scene.c.setBackground.connect(self._setBackground_)
        self.scene.c.crossSection.connect(self._crossSection_)
        self.scene.c.crossSectionRedraw.connect(self._crossSectionRedraw_)
        self.scene.c.mouseMove.connect(self._setMouseMovement_)
        self.sceneScale.c.scaleWarning.connect(self._scaleWarning_)
        self.sceneScale.c.scalebarChangePosition.connect(self._changeViewPosition_)
        self.scene.c.setWorking.connect(self.setStatusWorkingAndProcess)
        self.scene.c.setIdle.connect(self.setStatusIdleAndProcess)

        if (os.name != "nt"):
            self.btnRender.setVisible(False)
            self.ib3DThreshLower.setVisible(False)	
            self.ib3DThreshUpper.setVisible(False)
            self.lbl3DThreshlower.setVisible(False)
            self.lbl3DThreshUpper.setVisible(False)

        #self.movie = ImagePlayer(gif, "waiter")

    def setStatusWorking(self):
        self.lblWorking.setStyleSheet("color: rgb(255,0,0)")
        self.lblWorking.setText("Working...")

    def setStatusIdle(self):
        self.lblWorking.setStyleSheet("color: rgb(0,170,0)")
        self.lblWorking.setText("Idle...")

    def setStatusWorkingAndProcess(self):
        self.setStatusWorking()
        QtGui.QGuiApplication.processEvents()

    def setStatusIdleAndProcess(self):
        self.setStatusIdle()
        QtGui.QGuiApplication.processEvents()

    # image loading, modifying etc. -------------------------------------------------

    def _loadImageClicked_(self):
        self.setStatusWorking()
        try:
            self.gvScalebarTop.setStyleSheet("background: transparent")
            self.lblErrors.setText("")
            self.__qim__ = []

            filename = QtWidgets.QFileDialog.getOpenFileName(None, "Select Image", "", "Image Files(*.spe *.py *.tiff)")
            if filename[0] != "":
                if len(self.gvView.items()) != 0:
                    self.gvView.items().clear()
                    self.scene.clear()
                    self.scene.roi.clear()
                    self.scene.crossSections.clear()
                    self.scene.clearPixmaps()
                    self._refreshRoiAndStat_()
                    self.__refreshCrossStat__()
                    self.gvScalebarTop.items().clear()
                    self.useThresh = False

                self.data = self.__getData__(filename[0])

                # create image from spe data
                self.stack_size_sy = np.size(self.data, 1)
                self.stack_size_sx = np.size(self.data, 2)
                for d in self.data:
                    fig = plt.figure(frameon=False)
                    ax = plt.Axes(fig, [0., 0., 1., 1.])

                    ax.set_axis_off()
                    fig.add_axes(ax)
                    ax.imshow(d)
                    buf = io.BytesIO()
                    plt.savefig(buf, format='png', bbox_inches='tight', pad_inches=0)
                    plt.close(fig)
                    buf.seek(0)
                    im = Image.open(buf)
                    #im = self.__trim__(im)
                    img = im.convert("L")

                    # convert this image and save it into a qpixmap so it can we visualized in graphics view
                    d = im.tobytes("raw", "RGBA")
                    qim = QtGui.QImage(d, im.size[0], im.size[1], QtGui.QImage.Format_RGBA8888).scaled(self.stack_size_sx, self.stack_size_sy, QtCore.Qt.KeepAspectRatio)
                    self.__qim__.append(QtGui.QImage(d, im.size[0], im.size[1], QtGui.QImage.Format_ARGB32).scaled(self.stack_size_sx, self.stack_size_sy, QtCore.Qt.KeepAspectRatio))
                    pixColor = QtGui.QPixmap.fromImage(qim)

                    #[[im.putpixel((x, y), self.__getGrayValue__(im, x, y)) for y in range(0, im.height)] for x in
                    # range(0, im.width)]
                    #d = im.tobytes("raw", "RGBA")
                    #qimGray = QtGui.QImage(d, im.size[0], im.size[0], QtGui.QImage.Format_RGBA8888)
                    #pixGray = QtGui.QPixmap.fromImage(qimGray)

                    # gray version
                    d = img.tobytes("raw", "L")
                    qim = QtGui.QImage(d, im.size[0], im.size[1], QtGui.QImage.Format_Grayscale8)
                    pixGray = QtGui.QPixmap.fromImage(qim)


                    self.scene.addPixColorAt(pixColor, 0)
                    self.scene.addPixGrayAt(
                        pixGray.scaled(pixColor.width(), pixColor.height(), QtCore.Qt.KeepAspectRatio), 0)
                    buf.close()

                # create scene and load image into it
                self.scene.addText("")
                self.scene.addPixmap(pixColor)
                self.scene.setSceneRect(QtCore.QRectF(0, 0, pixColor.width(), pixColor.height()))
                self.gvView.setScene(self.scene)
                self.gvView.setSceneRect(self.scene.sceneRect())
                self.sceneScale.setSceneRect(self.scene.sceneRect())
                self.sceneWidth = self.scene.sceneRect().width()
                self.sceneHeight = self.scene.sceneRect().height()
                self.gvView.show()

                self.scene.setOrigHeight(self.scene)
                self.scene.setOrigWidth(pixColor.width())
                self.scene.setOrigHeight(pixColor.height())
                self.sceneScale.setImageCoords([QtCore.QPoint(0, 0), QtCore.QPoint(pixColor.width(), pixColor.height())])
                if len(self.data) == 1:
                    self.hSbImages.setMaximum(1)
                    self.hSbImages.setMinimum(1)
                    self.lblImageNr.setText("1\\1")
                else:
                    self.hSbImages.setValue(0)
                    self.hSbImages.setMaximum(len(self.data))
                    self.hSbImages.setMinimum(1)
                    self.lblImageNr.setText("1\\" + str(len(self.data)))
                    self.hSbImages.valueChanged.connect(self.__imgScaleBarChanged__)
                if self.cbScale.isChecked():
                    self.cbScale.setCheckState(False)
                    self.cbScale.setCheckState(True)
                self._changeViewClicked_()
                self.ibImgScale.setValue(1)
            else:
                self.lblErrors.setText("Invalid file, please try again!")
        except:
            self.lblErrors.setText("Invalid file, please try again!")
        self.setStatusIdle()

    def __imgScaleBarChanged__(self):
        i = self.hSbImages.value()-1
        self.lblImageNr.setText(str(i+1) + "\\" + str(len(self.data)))
        if not self.useThresh:
            if self.cbView.isChecked():
                self.scene.items()[len(self.scene.items()) - 2].setPixmap(self.scene.getPixGrayAt(i))
            else:
                self.scene.items()[len(self.scene.items()) - 2].setPixmap(self.scene.getPixColorAt(i))
        elif self.useThresh:
            self.scene.items()[len(self.scene.items()) - 2].setPixmap(self.scene.getPixThreshAt(i))
        self._refreshRoiAndStat_()
        self.__refreshCrossStat__()

    def getKey(self, l):
        return l[0]

    def _changeViewScale_(self):
        self._resetErrorMessage_()
        self.setStatusWorking()
        QtGui.QGuiApplication.processEvents()
        self.__changeVScale__()
        self.setStatusIdle()

    def __changeVScale__(self):
        scale = self.ibImgScale.value()
        self.ratio = scale
        self.sceneScale.positionScalebar()
        w = self.scene.getOrigWidth()
        h = self.scene.getOrigHeight()
        newcp = []
        newgp = []
        if self.useThresh:
            newtp = []
        for imagenr in range(0, len(self.data)):
            cp = self.scene.getPixColorAt(imagenr)
            gp = self.scene.getPixGrayAt(imagenr)
            if self.useThresh:
                tp = self.scene.getPixThreshAt(imagenr)
            newcp.append(cp.scaled(w * scale, h * scale, QtCore.Qt.KeepAspectRatio))
            newgp.append(gp.scaled(w * scale, h * scale, QtCore.Qt.KeepAspectRatio))
            if self.useThresh:
                newtp.append(tp.scaled(w * scale, h * scale, QtCore.Qt.KeepAspectRatio))
        self.scene.setPixColor(newcp)
        self.scene.setPixGray(newgp)
        if self.useThresh:
            self.scene.setPixThresh(newtp)
        self.scene.setSceneRect(QtCore.QRectF(0, 0, w * scale, h * scale))
        self.gvView.setSceneRect(self.scene.sceneRect())
        self._changeViewClicked_()
        self._redrawROI_()
        self._redrawCrossSec_()
        self.sceneScale.setImageCoords([QtCore.QPoint(0, 0), QtCore.QPoint(newcp[0].width(), newcp[0].height())])
        self.sceneScale.setRatio(self.ratio)
        self._redrawScaleBar_()
        for r in self.scene.roi:
            if r.getType() == ROI.ROIType.Polygon:
                self.__getPolyValues__(r)
            else:
                ori = r.getOriginalCoord()
                oriX = [ori[0].x(), ori[1].x()]
                oriY = [ori[0].y(), ori[1].y()]
                self.__getRectValues__([oriX[0], oriY[0]], [oriX[1], oriY[1]], r)
        self._RoiChangeClicked_()

    def _setMouseMovement_(self, x, y):
        c = self.__getOriginalCoord__([x, y])
        self.lblMousePos.setText("x = " + str(c[0]) + ", y = " + str(c[1]))

    def _changeViewClicked_(self):
        if self.data is not None:
            if self.useThresh == False:
                if self.cbView.isChecked():
                    self.scene.items()[len(self.scene.items()) - 2].setPixmap(self.scene.getPixGrayAt(self.hSbImages.value()-1))
                else:
                    self.scene.items()[len(self.scene.items()) - 2].setPixmap(self.scene.getPixColorAt(self.hSbImages.value()-1))
            elif self.useThresh == True:
                self.scene.items()[len(self.scene.items()) - 2].setPixmap(self.scene.getPixThreshAt(self.hSbImages.value()-1))
        else:
            self.lblErrors.setText("No data available! Please load in an image.")
            self.cbView.setChecked(False)

    def __getData__(self, filename):
        #DATA[Y][X]
        fn = filename.split('.')
        fn = fn[len(fn)-1]
        if (fn == "SPE" or fn == "spe"):
            data = read_spe2.load(filename)
            return [data]
        elif (fn == "PY" or fn == "py"):
            data = self.__read_py__(filename)
            return data
        else:
            data = self.__read_tiff__(filename)
            data = np.array(data, dtype=np.int32)
            return data

    def __read_py__(self, filename):
        filename = filename.split('/')
        self.filename = filename[len(filename)-1].split('.')[0]
        stack = importlib.import_module(self.filename)
        stack = stack.stack
        return stack

    def __read_tiff__(self, filename):
        im = tf.imread(filename)
        return [im]

    def _changeEdge_(self):
        self._resetErrorMessage_()
        if self.data is not None:
            lower = self.ibLower.value()
            upper = self.ibUpper.value()
            if lower < upper:
                img = qi2np.rgb_view(self.__qim__[self.__getScbVal__()])
                gray = cv.cvtColor(img, cv.COLOR_BGR2GRAY)
                f1 = plt.figure(1)
                plt.gray()
                plt.imshow(gray)
                edge = cv.Canny(gray, lower, upper)
                f2 = plt.figure(2)
                plt.gray()
                plt.imshow(edge)
                gray = cv.GaussianBlur(gray, (self.ibGDim.value(), self.ibGDim.value()), self.dbGSig.value())
                f3 = plt.figure(3)
                plt.gray()
                plt.imshow(gray)
                edge = cv.Canny(gray, lower, upper)
                f4 = plt.figure(4)
                plt.gray()
                plt.imshow(edge)
                plt.show()
            else:
                self.lblErrors.setText("Lower limit bigger than or equal to upper limit, please control your values!")
        else:
            self.lblErrors.setText("No data available! Please load in an image.")

    def _createSurfaceClicked_(self):
        self.lblErrors.setText("")
        self.__createSurface__()
        #self.create3D()

    def create3D(self):
        new = ([])
        for y in range(0, len(self.data)):
            for x in range(0, len(self.data[y])):
                k = self.data[y][0][x]
                new.append([x, y, k])

        v = np.array(new)
        fig = plt.figure()
        ax = fig.add_subplot(111, projection='3d')
        ax.scatter(v[:, 0], v[:, 1], v[:, 2])
        plt.show()

    def render3D(self):
        self.setStatusWorking()
        self._resetErrorMessage_()
        if (self.data is not None and len(self.data) > 1):
            volview.main(self.filename + ".py", self.ib3DThreshLower.value(), self.ib3DThreshUpper.value())
        else:
            self.lblErrors.setText("Unable to create 3D rendering! Image needs to have more than 1 layer.")
        self.setStatusWorking()

    def __createSurface__(self):
        self._resetErrorMessage_()
        if self.data is not None:
            # Set up grid and test data
            ny, nx = len(self.data[self.__getScbVal__()]), len(self.data[self.__getScbVal__()][0])
            x = range(nx)
            y = range(ny)

            hf = plt.figure()
            ha = hf.add_subplot(111, projection='3d')
            X, Y = np.meshgrid(x, y)  # `plot_surface` expects `x` and `y` data to be 2D
            plt.gca().invert_xaxis()
            d = self.data[self.__getScbVal__()]
            Z = np.array(d, dtype=np.int32)
            ha.plot_surface(X, Y, Z, cmap=cm.viridis, linewidth=0, antialiased=False)
            plt.xlabel("X")
            plt.ylabel("Y")
            plt.title("Surface Plot")
            plt.show()
        else:
            self.lblErrors.setText("No data available! Please load in an image.")

    def _createThreshClicked_(self):
        threshLimit = self.ibThresh.value()
        if threshLimit == -1:
            self.useThresh = False
            self._changeViewClicked_()
        else:
            self.__createThresholdImage__(threshLimit)

    def __createThresholdImage__(self, threshLimit):
        self._resetErrorMessage_()
        if self.data is not None:
            self.scene.pixThresh.clear()
            for imagenr in range (0, len(self.data)):
                img = qi2np.rgb_view(self.scene.getPixColorAt(imagenr).toImage().convertToFormat(QtGui.QImage.Format_ARGB32_Premultiplied))
                gray = cv.cvtColor(img, cv.COLOR_BGR2GRAY)
                gray[gray > threshLimit] = 255
                gray[gray <= threshLimit] = 0
                self.scene.appendPixThresh(QtGui.QPixmap.fromImage(qi2np.gray2qimage(gray)))
            self.scene.items()[len(self.scene.items()) - 2].setPixmap(self.scene.getPixThreshAt(self.hSbImages.value()-1))
            self.useThresh = True
            self._changeViewClicked_()
        else:
            self.lblErrors.setText("No data available! Please load in an image.")

    # scalebar--------------------------------------------------------------------
    def _changeViewPosition_(self):
        if self.sceneScale.scaleMode == ScaleMode.ScaleMode.BottomLeft or \
                self.sceneScale.scaleMode == ScaleMode.ScaleMode.BottomRight:
            h = self.gvView.height()
            if (self.scene.getOrigHeight()*self.ratio) <= h:
                self.gvScalebarTop.setGeometry(10, self.scene.getOrigHeight()*self.ratio + self.gvView.geometry().y(), self.gvScalebarTop.width(),
                                               self.gvScalebarTop.height())
            else:
                y = self.gvView.height() + self.gvView.geometry().topLeft().y()
                self.gvScalebarTop.setGeometry(10, y, self.gvScalebarTop.width(),
                                               self.gvScalebarTop.height())
        else:
            self.gvScalebarTop.setGeometry(10, 30, self.gvScalebarTop.width(), self.gvScalebarTop.height())

    def _scaleWarning_(self):
        self.lblErrors.setText("Scale too big! Please select a smaller one.")

    def _scalePosChange_(self):
        self.sceneScale.setScaleMode(self.cbScalePos.currentIndex())
        self._redrawScaleBar_()

    def _scaleValuesChanged_(self):
        self._resetErrorMessage_()
        if self.cbScale.isChecked():
            self.sceneScale.removeScale()
            self.sceneScale.setScale(self.dbScale.value())
            self.sceneScale.setScaleLen(self.ibScaleLength.value())
            self.sceneScale.createScale(True)

    def _redrawScaleBar_(self):
        if self.cbScale.isChecked():
            self.sceneScale.redrawScalebar()

    def _ScaleVisibilityClicked_(self):
        if self.cbScale.isChecked():
            self.sceneScale.setScale(self.dbScale.value())
            self.sceneScale.setScaleLen(self.ibScaleLength.value())
            self.sceneScale.createScale(self.cbScale.isChecked())
        else:
            self.sceneScale.removeScale()

    # button action----------------------------------------------------------------
    def _setButtonAction_(self):
        # background
        self.btnSetBackground.clicked.connect(self._setBackgroundClicked_)
        # button actions
        self.btnLoadImage.clicked.connect(self._loadImageClicked_)
        # ROI
        self.btnAddROI.clicked.connect(self._addClicked_)
        self.btnDeleteROI.clicked.connect(self._deleteClicked_)
        self.btnUp.clicked.connect(self._shiftUpClicked_)
        self.btnDown.clicked.connect(self._shiftDownClicked_)
        self.btnLeft.clicked.connect(self._shiftLeftClicked_)
        self.btnRight.clicked.connect(self._shiftRightClicked_)
        self.btnCopyROI.clicked.connect(self._copyROIClicked_)
        self.btnDetectROI.clicked.connect(self._detectROIClicked_)
        # tables
        self.tblAvgStat.itemSelectionChanged.connect(self._RoiChangeClicked_)

        # combobox
        self.cbROI.currentIndexChanged.connect(self._cbChange_)
        self.cbScalePos.currentIndexChanged.connect(self._scalePosChange_)
        # load and save
        self.btnSaveROI.clicked.connect(self._saveROIClicked_)
        self.btnLoadROI.clicked.connect(self._loadROIClicked_)
        self.btnHist.clicked.connect(self._createHistClicked_)
        self.btnSurface.clicked.connect(self._createSurfaceClicked_)
        self.btnExport.clicked.connect(self._exportStatClicked_)
        # tabs
        self.cbView.stateChanged.connect(self._changeViewClicked_)
        self.cbScale.stateChanged.connect(self._ScaleVisibilityClicked_)
        # Cross Section
        self.btnAddCrossSec.clicked.connect(self._crossSectionClicked_)
        self.btnCreateCrossChart.clicked.connect(self._createCrossSecPlotClicked_)
        self.btnDeleteCrossSec.clicked.connect(self._deleteCrossSectionClicked_)

        self.btnCanny.clicked.connect(self._changeEdge_)
        self.cbScale.clicked.connect(self._setCbScaleErrorMessage_)


        self.dbScale.valueChanged.connect(self._scaleValuesChanged_)

        self.ibImgScale.valueChanged.connect(self._changeViewScale_)
        self.ibScaleLength.valueChanged.connect(self._scaleValuesChanged_)

        self.btnThresh.clicked.connect(self._createThreshClicked_)

        self.btnShowAxis.clicked.connect(self._showAxis_)

        self.btnRender.clicked.connect(self.render3D)

        self.cnt = 0
        self.cntCross = 0

    def _showAxis_(self):
        self._resetErrorMessage_()
        self.setStatusWorking()
        QtGui.QGuiApplication.processEvents()
        if self.data is not None:
            self._resetErrorMessage_()
            if len(self.data) > 1:
                self.dialog = QtWidgets.QDialog()
                self.dialog.ui = StackDataGUI(Ui_Dialog)
                self.dialog.ui.setupUi(self.dialog)
                self.dialog.ui.initialize()
                self.dialog.ui.readData(self.data)
                self.dialog.setModal(True)
                self.dialog.open()
            else:
                self.lblErrors.setText("Could not display axis, since this is a two dimensional and not three dimensional Image. ")
        else:
            self.lblErrors.setText("No data available! Please load in an image.")
        self.setStatusIdle()

    def _setCbScaleErrorMessage_(self):
        self._resetErrorMessage_()
        if self.data is None:
            self.lblErrors.setText("No data available! Please load in an image.")
            self.cbScale.setChecked(False)

    # cross section----------------------------------------------------------------
    def _crossSection_(self):
        self.setStatusWorking()
        QtGui.QGuiApplication.processEvents()
        coord = self.scene.getCrossSecCoord()
        if len(coord) == 4:
            coord1 = self.__getOriginalCoord__(coord[:2])
            coord2 = self.__getOriginalCoord__(coord[2:])
            latestCross = self.scene.getLatestLine()
            cnt = 0
            for i in range(0, len(self.data)):
                v = []
                if coord1[0] == coord2[0]:
                    if coord1[1] > coord2[1]:
                        for y in range(coord2[1], coord1[1]):
                            v.append(self.data[1][y][coord1[0]])
                            if i == 0:
                                latestCross.appendxAxis(cnt)
                                cnt = cnt + 1
                    elif coord1[1] < coord2[1]:
                        for y in range(coord1[1], coord2[1]):
                            v.append(self.data[i][y][coord1[0]])
                            if i == 0:
                                latestCross.appendxAxis(cnt)
                                cnt = cnt + 1
                elif coord1[1] == coord2[1]:
                    for x in range(min([coord2[0], coord1[0]]), max([coord2[0], coord1[0]])):
                        v.append(self.data[i][coord1[1]][x])
                        if i == 0:
                            latestCross.appendxAxis(cnt)
                            cnt = cnt + 1
                else:
                    li = self.scene.getLatestLine().getLine()
                    rect = li.boundingRect()
                    if rect.topLeft().x() != rect.bottomRight().x():
                        for x in range(int(rect.topLeft().x()), int(rect.bottomRight().x())):
                            for y in range(int(rect.topLeft().y()), int(rect.bottomRight().y())):
                                if li.contains(QtCore.QPointF(x, y)):
                                    c = self.__getOriginalCoord__([x, y])
                                    if "[" + str(c[1]) + "," + str(c[1]) + "]" not in latestCross.getxAxis():
                                        v.append(self.data[i][int(np.round(c[1]))][int(np.round(c[0]))])
                                        if i == 0:
                                            latestCross.appendxAxis(cnt)
                                            cnt = cnt + 1
                    else:
                        for y in range(int(rect.topLeft().y()), int(rect.bottomRight().y())):
                            for x in range(int(rect.topLeft().x()), int(rect.bottomRight().x())):
                                if li.contains(QtCore.QPointF(x, y)):
                                    c = self.__getOriginalCoord__([x, y])
                                    if "[" + str(c[1]) + "," + str(c[1]) + "]" not in latestCross.getxAxis():
                                        v.append(self.data[i][c[1]][c[0]])
                                        if i == 0:
                                            latestCross.appendxAxis(cnt)
                                            cnt = cnt + 1
                latestCross.appendPixelVal(v)
            self.__refreshCrossStat__()
            self.scene.cross.clear()
            self.scene.crossSec = False
            self.setStatusIdle()

    def _crossSectionRedraw_(self):
        for c in self.scene.getCrossSections():
            li = c.getOriginalLine()
            rect = li.boundingRect()
            cnt = 0
            for i in range(0, len(self.data)):
                v = []
                for x in range(int(rect.topLeft().x()), int(rect.bottomRight().x())):
                    for y in range(int(rect.topLeft().y()), int(rect.bottomRight().y())):
                        if li.contains(QtCore.QPointF(x, y)):
                            if "[" + str(x) + "," + str(y) + "]" not in c.getxAxis():
                                #c.appendPixelVal(self.data[self.__getScbVal__()][y][x])
                                v.append(self.data[self.__getScbVal__()][y][x])
                                if i == 0:
                                    c.appendxAxis(cnt)
                                    cnt = cnt + 1
                c.appendPixelVal(v)
            # print("hi")

    def _redrawCrossSec_(self):
        self.scene.redrawCrossSec()

    def _createCrossSecPlotClicked_(self):
        self._resetErrorMessage_()
        if self.data is not None:
            rowNr = self.tblCrossSec.currentRow()
            if rowNr >= 0 and self.twViewStat.currentIndex() == 0:
                roirow = int(self.tblCrossSec.item(rowNr, 0).text())
                self._createCrossSecPlot_(self.scene.crossSections[roirow], roirow)
            else:
                self.lblErrors.setText("Could not create Cross Section Plot! Please be sure to select the \"View\" Tab " +
                                       "and an element from the \"Cross Sections\" table.")
        else:
            self.lblErrors.setText("No data available! Please load in an image.")

    def _createCrossSecPlot_(self, crossSec, roirow):
        plt.figure(self.cntCross)
        self.cntCross = self.cntCross + 1
        plt.grid(color='grey', linestyle=':', linewidth=1)
        val = crossSec.getPixelValAt(self.__getScbVal__())
        plt.plot(crossSec.getxAxis(), val)
        plt.xlabel("Coordinate")
        plt.ylabel("Intensity")
        plt.title("Cross Section " + str(roirow) + " Layer " + str(self.__getScbVal__()+1))
        plt.show()

    def _deleteCrossSectionClicked_(self):
        self._resetErrorMessage_()
        if self.data is not None:
            if self.twViewStat.currentIndex() == 1:
                self.lblErrors.setText("Could not delete Cross Section! Please be sure to select the \"View\" Tab " +
                                       "and an element from the \"Cross Sections\" table.")
            else:
                rowNr = self.tblCrossSec.currentRow()
                if rowNr >= 0:
                    roirow = int(self.tblCrossSec.item(rowNr, 0).text())
                    self.scene.removeItem(self.scene.crossSections[roirow].getLine())
                    self.scene.crossSections.pop(roirow)
                    self.__refreshCrossStat__()
                else:
                    self.lblErrors.setText("Could not delete Cross Section! Please be sure to select the \"View\" Tab " +
                                           "and an element from the \"Cross Sections\" table.")
        else:
            self.lblErrors.setText("No data available! Please load in an image.")

    def _crossSectionClicked_(self):
        self._resetErrorMessage_()
        if self.data is not None:
            if self.twViewStat.currentIndex() == 1:
                self.lblErrors.setText("Could not create Cross Section! Please be sure to select the \"View\" Tab.")
            else:
                self.scene.crossSec = True
        else:
            self.lblErrors.setText("No data available! Please load in an image.")

    def _tblCrossChangeClicked_(self):
        rowNr = self.tblCrossSec.currentRow()
        if rowNr >= 0:
            roirow = int(self.tblCrossSec.item(rowNr, 0).text())
            items = self.scene.items()[:-2]
            items = items[::-1]
            for i in range(0, len(items)):
                if i == roirow and items[i].type() == 6:
                    items[i].setOpacity(1)

                elif items[i].type() == 6:
                    items[i].setOpacity(0.5)

    def __refreshCrossStat__(self):
        self.tblCrossSec = self.__clearTable__(self.tblCrossSec)
        self.tblCrossSec.setSortingEnabled(False)
        sec = self.scene.getCrossSections()
        for x in range(0, len(sec)):
            val = sec[x].getPixelValAt(self.__getScbVal__())
            row = self.tblCrossSec.rowCount()
            self.tblCrossSec.insertRow(row)
            coords = sec[x].getOriginalCoord()
            c = "[(" + str(coords[0].x()) + "," +str(coords[0].y()) + "),(" + str(coords[1].x()) + "," +str(coords[1].y()) + ")]"
            item1 = QtWidgets.QTableWidgetItem()
            item1.setData(QtCore.Qt.EditRole, str(x))
            item2 = QtWidgets.QTableWidgetItem()
            item2.setData(QtCore.Qt.EditRole, str(c))
            item3 = QtWidgets.QTableWidgetItem()
            item3.setData(QtCore.Qt.EditRole, int(sec[x].getLength()))
            item4 = QtWidgets.QTableWidgetItem()
            item4.setData(QtCore.Qt.EditRole, int(min(val)))
            item5 = QtWidgets.QTableWidgetItem()
            item5.setData(QtCore.Qt.EditRole, float(np.round(sum(val) / sec[x].getPixelValLenAt(self.__getScbVal__()), 4)))
            item6 = QtWidgets.QTableWidgetItem()
            item6.setData(QtCore.Qt.EditRole, int(max(val)))

            self.tblCrossSec.setItem(row, 0, item1)
            self.tblCrossSec.setItem(row, 1, item2)
            self.tblCrossSec.setItem(row, 2, item3)
            self.tblCrossSec.setItem(row, 3, item4)
            self.tblCrossSec.setItem(row, 4, item5)
            self.tblCrossSec.setItem(row, 5, item6)
        self.tblCrossSec.setSortingEnabled(True)

    # ROI----------------------------------------------------------------------
    def _redrawROI_(self):
        #self.setStatusWorking()
        self.scene.redrawROI(self.ratio)
        #self.setStatusIdle()

    def _refreshRoiAndStat_(self):
        self._refreshStats_()

    def _createHistClicked_(self):
        self._resetErrorMessage_()
        if self.data is not None:
            rowNr = self.tblAvgStat.currentRow()
            if rowNr >= 0 and self.twViewStat.currentIndex() == 1:
                roirow = int(self.tblAvgStat.item(rowNr, 0).text())
                self.__createHist__(roirow)
            else:
                self.lblErrors.setText("Could not create Histogram! Please be sure to select the \"Statistics\" Tab " +
                                       "and an element from the \"ROI Statistics\" table.")
        else:
            self.lblErrors.setText("No data available! Please load in an image.")

    def _saveROIClicked_(self):
        self.lblErrors.setText("")
        if self.data is not None:
            if self.twViewStat.currentIndex() == 0:
                rowNr = self.tblROI.currentRow()
            else:
                rowNr = self.tblROITabStat.currentRow()
            if rowNr >= 0:
                if self.twViewStat.currentIndex() == 0:
                    roirow = int(self.tblROI.item(rowNr, 0).text())
                else:
                    roirow = int(self.tblROITabStat.item(rowNr, 0).text())
                singleroi = self.scene.roi[roirow]
                filename = QtWidgets.QFileDialog.getOpenFileName(None, "Select File", "", "Files(*.txt)")
                if filename[0] != "":
                    with open(filename[0], "w") as f:
                        if singleroi.RoiType == ROI.ROIType.Rectangle:
                            spots = singleroi.getOriginalCoord()
                            top = str(int(spots[0].x())) + " , " + str(int(spots[0].y()))
                            bottom = str(int(spots[1].x())) + " , " + str(int(spots[1].y()))
                            t = singleroi.RoiType.name

                            f.write(t + "\n" + top + "\n" + bottom + "\n")
                        else:
                            f.write(singleroi.RoiType.name + "\n")
                            for i in range(0, self.scene.roi[roirow].getOriginalPoly().polygon().count()):
                                p = self.scene.roi[roirow].getOriginalPoly().polygon().at(i)
                                f.write(str(int(p.x())) + "," + str(int(p.y())) + "\n")
                else:
                    self.lblErrors.setText("Could not save ROI, because of an invalid filename.")
            else:
                self.lblErrors.setText("Could not save ROI! Please be sure to select an element from \"ROIs\".")
        else:
            self.lblErrors.setText("No data available! Please load in an image.")

    def _loadROIClicked_(self):
        self._resetErrorMessage_()
        self.setStatusWorking()
        QtGui.QGuiApplication.processEvents()
        if self.data is not None:
            filename = QtWidgets.QFileDialog.getOpenFileName(None, "Select File", "", "Files(*.txt)")
            if filename[0] != "":
                with open(filename[0], "r") as f:
                    points = []
                    for l in f:
                        p = l.strip("\n")
                        p = p.split(",")
                        points.append(p)
                    w = int(points[1][0]) * self.ratio >= 0 and (int(points[2][0]) * self.ratio) < len(self.data[0][0])
                    h = int(points[1][1]) * self.ratio >= 0 and (int(points[2][1]) * self.ratio) < len(self.data[0])
                    if w and h:
                        if points[0][0] == ROI.ROIType.Rectangle.name:
                            self.scene.addRectangle(
                                QtCore.QPointF(int(points[1][0]) * self.ratio, int(points[1][1]) * self.ratio),
                                QtCore.QPointF(int(points[2][0]) * self.ratio, int(points[2][1]) * self.ratio))
                            self.scene.roi[len(self.scene.roi) - 1].setOriginalCoord(
                                [QtCore.QPointF(int(points[1][0]), int(points[1][1])),
                                 QtCore.QPointF(int(points[2][0]), int(points[2][1]))])
                            self.scene.roi[len(self.scene.roi) - 1].adjustAfterSetOriginal()
                        elif points[0][0] == ROI.ROIType.Polygon.name:
                            self.scene.addPoly(points, 0, 0)
                    else:
                        self.lblErrors.setText("Invalid ROI coordinates, please check your file.")
                self._refreshStats_()
                self._refreshROIs_()
            else:
                self.lblErrors.setText("Invalid file, please try again.")
        else:
            self.lblErrors.setText("No data available! Please load in an image.")
        self.setStatusIdle()

    def _addClicked_(self):
        self._resetErrorMessage_()
        self.setStatusWorking()
        QtGui.QGuiApplication.processEvents()
        if self.data is not None:
            if self.twViewStat.currentIndex() == 0:
                self.scene.add = True
                self.scene.scale = self.ratio
                self.scene.setbackground = False
                self.scene.backgroundCoord.clear()
                self.scene.rect = str(self.cbROI.currentText()) == "Rectangle"
            else:
                self.lblErrors.setText("Could not create ROI! Please be sure to select the \"View\" Tab")
        else:
            self.lblErrors.setText("No data available! Please load in an image.")
        self.setStatusIdle()

    def _deleteClicked_(self):
        self.lblErrors.setText("")
        if self.data is not None:
            if self.twViewStat.currentIndex() == 0:
                rowNr = self.tblROI.currentRow()
            else:
                rowNr = self.tblROITabStat.currentRow()
            if rowNr >= 0:
                self.lblErrors.setText("")
                if self.twViewStat.currentIndex() == 0:
                    roirow = int(self.tblROI.item(rowNr, 0).text())
                else:
                    roirow = int(self.tblROITabStat.item(rowNr, 0).text())
                if self.scene.roi[roirow].getType() is ROI.ROIType.Rectangle:
                    self.scene.removeItem(self.scene.roi[roirow].getRect())
                else:
                    self.scene.removeItem(self.scene.roi[roirow].getPoly())
                self.tblROI.removeRow(roirow)
                self.tblAvgStat.removeRow(roirow)
                self.scene.roi.pop(roirow)
                self._refreshROIs_()
                self._refreshStats_()
            else:
                self.lblErrors.setText("Could not delete ROI! Please be sure to select an element from the \"ROIs\" table.")
        else:
            self.lblErrors.setText("No data available! Please load in an image.")

    def _shiftUpClicked_(self):
        self._resetErrorMessage_()
        self.setStatusWorking()
        QtGui.QGuiApplication.processEvents()
        if self.data is not None:
            offset = self.dbShift.value()
            rowNr = self.tblROI.currentRow()
            if rowNr >= 0 and self.twViewStat.currentIndex() == 0:
                self.lblErrors.setText("")
                roirow = int(self.tblROI.item(rowNr, 0).text())
                singleROI = self.scene.roi[roirow]
                if self.__validShift__(singleROI, - offset):
                    if singleROI.getType() == ROI.ROIType.Rectangle:
                        y = singleROI.getTopLeft().y() - (offset * self.ratio)
                        r = QtCore.QRectF(singleROI.getTopLeft().x(), y, singleROI.getSceneWidth(),
                                          singleROI.getSceneHeight())
                        singleROI.getRect().setRect(r)
                        singleROI.shiftY(-offset)
                    elif singleROI.getType() == ROI.ROIType.Polygon:
                        poly = QtGui.QPolygonF()
                        scenePoly = singleROI.getPoly().polygon()
                        Parallel(n_jobs=self.num_cores, prefer="threads")(
                            delayed(self.__doPolyAddY_)(scenePoly, poly, -offset * self.ratio, i) for i in range(0, scenePoly.count()))
                        singleROI.getPoly().setPolygon(poly)
                        singleROI.shiftY(-offset)
                    self._refreshRoiAndStat_()
                    self.tblROI.selectRow(rowNr)
                else:
                    self.lblErrors.setText("Could not shift ROI! Shifting would have caused the ROI to leave the Image, " +
                                           "please check your offset.")
            else:
                self.lblErrors.setText("Could not shift ROI! Please be sure to select the \"View\" Tab and an element " +
                                       "from the \"ROIs\" table.")
        else:
            self.lblErrors.setText("No data available! Please load in an image.")
        self.setStatusIdle()

    def _shiftDownClicked_(self):
        self._resetErrorMessage_()
        self.setStatusWorking()
        QtGui.QGuiApplication.processEvents()
        if self.data is not None:
            offset = self.dbShift.value()
            rowNr = self.tblROI.currentRow()
            if rowNr >= 0 and self.twViewStat.currentIndex() == 0:
                self.lblErrors.setText("")
                roirow = int(self.tblROI.item(rowNr, 0).text())
                singleROI = self.scene.roi[roirow]
                if self.__validShift__(singleROI, offset):
                    if singleROI.getType() == ROI.ROIType.Rectangle:
                        y = singleROI.getTopLeft().y() + (offset * self.ratio)
                        r = QtCore.QRectF(singleROI.getTopLeft().x(), y, singleROI.getSceneWidth(),
                                          singleROI.getSceneHeight())
                        singleROI.setRect(r)
                        singleROI.shiftY(offset)
                    else:
                        poly = QtGui.QPolygonF()
                        scenePoly = singleROI.getPoly().polygon()
                        Parallel(n_jobs=self.num_cores, prefer="threads")(
                            delayed(self.__doPolyAddY_)(scenePoly, poly, offset * self.ratio, i) for i in range(0, scenePoly.count()))
                        singleROI.getPoly().setPolygon(poly)
                        singleROI.shiftY(offset)
                else:
                    self.lblErrors.setText("Could not shift ROI! Shifting would have caused the ROI to leave the Image, " +
                                            "please check your offset.")

                self._refreshRoiAndStat_()
                self.tblROI.selectRow(rowNr)
            else:
                self.lblErrors.setText("Could not shift ROI! Please be sure to select the \"View\" Tab and an element" +
                                       " from the \"ROIs\" table.")
        else:
            self.lblErrors.setText("No data available! Please load in an image.")
        self.setStatusIdle()

    def _shiftLeftClicked_(self):
        self._resetErrorMessage_()
        self.setStatusWorking()
        QtGui.QGuiApplication.processEvents()
        if self.data is not None: 
            offset = self.dbShift.value()
            rowNr = self.tblROI.currentRow()
            if rowNr >= 0 and self.twViewStat.currentIndex() == 0:
                self.lblErrors.setText("")
                roirow = int(self.tblROI.item(rowNr, 0).text())
                singleROI = self.scene.roi[roirow]
                if self.__validShift__(singleROI, -offset, True):
                    if singleROI.getType() == ROI.ROIType.Rectangle:
                        x = singleROI.getTopLeft().x() - (offset * self.ratio)
                        r = QtCore.QRectF(x, singleROI.getTopLeft().y(), singleROI.getSceneWidth(),
                                          singleROI.getSceneHeight())
                        singleROI.setRect(r)
                        singleROI.shiftX(-offset)
                    else:
                        poly = QtGui.QPolygonF()
                        scenePoly = singleROI.getPoly().polygon()
                        Parallel(n_jobs=self.num_cores, prefer="threads")(
                            delayed(self.__doPolyAddX_)(scenePoly, poly, - offset * self.ratio, i) for i in range(0, scenePoly.count()))
                        singleROI.getPoly().setPolygon(poly)
                        singleROI.shiftX(-offset)
                    self._refreshRoiAndStat_()
                    self.tblROI.selectRow(rowNr)
                else:
                    self.lblErrors.setText("Could not shift ROI! Shifting would have caused the ROI to leave the Image, " +
                                       "please check your offset.")
            else:
                self.lblErrors.setText("Could not shift ROI! Please be sure to select the \"View\" Tab and an " +
                                       "element from the \"ROIs\" table.")
        else:
            self.lblErrors.setText("No data available! Please load in an image.")
        self.setStatusIdle()

    def _shiftRightClicked_(self):
        self._resetErrorMessage_()
        self.setStatusWorking()
        QtGui.QGuiApplication.processEvents()
        if self.data is not None:
            offset = self.dbShift.value()
            rowNr = self.tblROI.currentRow()
            if rowNr >= 0 and self.twViewStat.currentIndex() == 0:
                self.lblErrors.setText("")
                roirow = int(self.tblROI.item(rowNr, 0).text())
                singleROI = self.scene.roi[roirow]
                if self.__validShift__(singleROI, offset, True):
                    if singleROI.getType() == ROI.ROIType.Rectangle:
                        x = singleROI.getTopLeft().x() + (offset * self.ratio)
                        r = QtCore.QRectF(x, singleROI.getTopLeft().y(), singleROI.getSceneWidth(),
                                          singleROI.getSceneHeight())
                        singleROI.setRect(r)
                        singleROI.shiftX(offset)
                    else:
                        poly = QtGui.QPolygonF()
                        scenePoly = singleROI.getPoly().polygon()
                        Parallel(n_jobs=self.num_cores, prefer="threads")(
                            delayed(self.__doPolyAddX_)(scenePoly, poly, offset*self.ratio, i) for i in range(0, scenePoly.count()))
                        singleROI.getPoly().setPolygon(poly)
                        singleROI.shiftX(offset)
                    self._refreshRoiAndStat_()
                    self.tblROI.selectRow(rowNr)
                else:
                    self.lblErrors.setText("Could not shift ROI! Shifting would have caused the ROI to leave the Image, " +
                                       "please check your offset.")
            else:
                self.lblErrors.setText("Could not shift ROI! Please be sure to select the \"View\" Tab and " +
                                       "an element from the \"ROIs\" table.")
        else:
            self.lblErrors.setText("No data available! Please load in an image.")
        self.setStatusIdle()

    def __validShift__(self, singleROI, offset, xAxis = False):
        if xAxis:
            newX = [x + (offset * self.ratio) for x in singleROI.getX()]
            newY = singleROI.getY()
        else:
            newX = singleROI.getX()
            newY = [y + (offset * self.ratio) for y in singleROI.getY()]
        if self.ratio != 1 and (xAxis and newX[0] == -1 and newX[1] < len(self.data[0][0])*self.ratio) or \
                (xAxis is False and newY[0] == -1 and newY[1] < len(self.data[0])*self.ratio):
            return True
        return (newX[0] >= 0 and newX[1] < len(self.data[0][0])*self.ratio) and (newY[0] >= 0 and newY[1] < len(self.data[0])*self.ratio)

    def _copyROIClicked_(self):
        self._resetErrorMessage_()
        if self.data is not None:
            self.setStatusWorking()
            QtGui.QGuiApplication.processEvents()
            rowNr = self.tblROI.currentRow()
            if rowNr >= 0 and self.twViewStat.currentIndex() == 0:
                self.lblErrors.setText("")
                roirow = int(self.tblROI.item(rowNr, 0).text())
                if self.scene.roi[roirow].getType() == ROI.ROIType.Rectangle:
                    rect = self.scene.roi[roirow].getRect().rect()
                    x1 = rect.topLeft().x()
                    y1 = rect.topLeft().y()
                    width = rect.width()
                    height = rect.height()
                    roi = ROI.RectROI(
                        self.scene.addRect(QtCore.QRectF(x1, y1, width, height), self.scene.pen, self.scene.brush))
                    roi.setHeight(self.scene.roi[roirow].getHeight())
                    roi.setWidth(self.scene.roi[roirow].getWidth())
                    roi.setOriginalCoord((self.scene.roi[roirow].getOriginalCoord()))
                    roi.setupdateNeeded(True)
                else:
                    poly = QtGui.QPolygonF()
                    for i in range(0, self.scene.roi[roirow].getPoly().polygon().count()):
                        p = self.scene.roi[roirow].getPoly().polygon().at(i)
                        poly.append(QtCore.QPoint(p.x(), p.y()))
                    roi = ROI.PolyROI(self.scene.addPolygon(poly, self.scene.pen, self.scene.brush), self.ratio)
                    roi.setAfterCopy(self.ratio)
                    roi.adjustAfterSet()
                    roi.setupdateNeeded(True)
                self.scene.roi.append(roi)
                self._refreshRoiAndStat_()
            else:
                self.lblErrors.setText("Could not copy ROI! Please be sure to select the \"View\" Tab and an " +
                                       "element from the \"ROIs\" table.")
            self.setStatusIdle()
        else:
            self.lblErrors.setText("No data available! Please load in an image.")

    def _refreshROIs_(self):
        self.tblROI = self.__clearTable__(self.tblROI)
        self.tblROI.setSortingEnabled(False)
        for x in range(0, len(self.scene.roi)):
            row = self.tblROI.rowCount()
            self.tblROI.insertRow(row)
            singleRoi = self.scene.roi[row]

            item1 = QtWidgets.QTableWidgetItem()
            item1.setData(QtCore.Qt.EditRole, row)
            item2 = QtWidgets.QTableWidgetItem()
            item2.setData(QtCore.Qt.EditRole, float(singleRoi.getWidth()))
            item3 = QtWidgets.QTableWidgetItem()
            item3.setData(QtCore.Qt.EditRole, float(singleRoi.getHeight()))
            item6 = QtWidgets.QTableWidgetItem()
            item6.setData(QtCore.Qt.EditRole, int(singleRoi.getArea()))
            x = np.round(singleRoi.getX(), 1)
            y = np.round(singleRoi.getY(), 1)
            self.tblROI.setItem(row, 0, item1)
            self.tblROI.setItem(row, 1, item2)
            self.tblROI.setItem(row, 2, item3)
            self.tblROI.setItem(row, 3, QtWidgets.QTableWidgetItem(str(x)))
            self.tblROI.setItem(row, 4, QtWidgets.QTableWidgetItem(str(y)))
            self.tblROI.setItem(row, 5, item6)
        self.tblROI.setSortingEnabled(True)

    def _refreshROIsTabStat_(self):
        self.tblROITabStat = self.__clearTable__(self.tblROITabStat)
        self.tblROITabStat.setSortingEnabled(False)
        for x in range(0, len(self.scene.roi)):
            row = self.tblROITabStat.rowCount()
            self.tblROITabStat.insertRow(row)
            singleRoi = self.scene.roi[row]

            item1 = QtWidgets.QTableWidgetItem()
            item1.setData(QtCore.Qt.EditRole, row)
            item2 = QtWidgets.QTableWidgetItem()
            item2.setData(QtCore.Qt.EditRole, int(singleRoi.getWidth()))
            item3 = QtWidgets.QTableWidgetItem()
            item3.setData(QtCore.Qt.EditRole, int(singleRoi.getHeight()))
            item6 = QtWidgets.QTableWidgetItem()
            item6.setData(QtCore.Qt.EditRole, int(singleRoi.getArea()))

            x = np.round(singleRoi.getX(), 1)
            y = np.round(singleRoi.getY(), 1)
            self.tblROITabStat.setItem(row, 0, item1)
            self.tblROITabStat.setItem(row, 1, item2)
            self.tblROITabStat.setItem(row, 2, item3)
            self.tblROITabStat.setItem(row, 3, QtWidgets.QTableWidgetItem(str(x)))
            self.tblROITabStat.setItem(row, 4, QtWidgets.QTableWidgetItem(str(y)))
            self.tblROITabStat.setItem(row, 5, item6)
        self.tblROITabStat.setSortingEnabled(True)

    def _refreshStats_(self):
        self._refreshROIs_()
        self._refreshROIsTabStat_()
        self.tblROIStatistic = self.__clearTable__(self.tblROIStatistic)
        self.tblAvgStat = self.__clearTable__(self.tblAvgStat)
        self.tblAvgStat.setSortingEnabled(False)
        self.tblROIStatistic.setSortingEnabled(False)
        valid = True

        for x in range(0, len(self.scene.roi)):
            singleRoi = self.scene.roi[x]
            if singleRoi.getType() == ROI.ROIType.Rectangle and min(singleRoi.getY()) >= 0 and \
                    min(singleRoi.getX()) >= 0 and max(singleRoi.getY()) <= self.sceneWidth and \
                    max(singleRoi.getX()) <= self.sceneHeight:
                coords1 = self.__getOriginalCoord__(
                    [singleRoi.getTopLeft().x(), singleRoi.getTopLeft().y()])
                coords2 = self.__getOriginalCoord__([singleRoi.getBottomRight().x(), singleRoi.getBottomRight().y()])
                if singleRoi.getUpdated():
                    self.__getRectValues__(coords1, coords2, self.scene.roi[x])
            elif singleRoi.getType() == ROI.ROIType.Polygon and min(singleRoi.getY()) >= 0 and min(
                    singleRoi.getX()) >= 0 and max(singleRoi.getY()) <= self.sceneWidth and max(
                        singleRoi.getX()) <= self.sceneHeight:
                if singleRoi.getUpdated():
                    self.__getPolyValues__(singleRoi)
            else:
                valid = False
            self.__refreshAvgStat__(singleRoi, x, valid)
        self.tblAvgStat.setSortingEnabled(True)
        self.tblROIStatistic.setSortingEnabled(True)

    def _RoiChangeClicked_(self):
        rowNr = self.tblAvgStat.currentRow()
        if rowNr >= 0:
            roirow = int(self.tblAvgStat.item(rowNr, 0).text())
            self.tblROIStatistic = self.__clearTable__(self.tblROIStatistic)
            roi = self.scene.roi[roirow]
            self.__refreshPixelStat__(roi, roirow)

    def _tblRoiChangeClicked_(self):
        rowNr = self.tblROI.currentRow()
        if rowNr >= 0:
            roirow = int(self.tblROI.item(rowNr, 0).text())
            items = self.scene.items()[:-2]
            items = items[::-1]
            for i in range(0, len(items)):
                if i == roirow and items[i].type() != 6:
                    items[i].setOpacity(1)
                elif items[i].type() != 6:
                    items[i].setOpacity(0.8)

    def __createHist__(self, roirow):
        plt.figure(self.cnt)
        imgC = self.__getScbVal__()
        values = self.scene.roi[roirow].getPixelsAt(imgC)
        plt.hist(x=values, color="black")
        plt.xlabel("Intensity")
        plt.ylabel("Number of Pixel")
        if len(self.data) > 1:
            plt.title("Layer " + str(self.__getScbVal__()+1) + " ROI " + str(roirow))
        else:
            plt.title("ROI " + str(roirow))
        plt.grid(True)
        plt.plot(figsize=(10, 5))
        plt.show()
        self.cnt = self.cnt + 1

    def __refreshAvgStat__(self, roi, x, valid):
        row = self.tblAvgStat.rowCount()
        if valid:
            values = roi.getPixelsAt(self.__getScbVal__())
            self.tblAvgStat.insertRow(row)
            if len(values) > 0:
                item1 = QtWidgets.QTableWidgetItem()
                item1.setData(QtCore.Qt.EditRole, row)
                item2 = QtWidgets.QTableWidgetItem()
                item2.setData(QtCore.Qt.EditRole, x)
                item3 = QtWidgets.QTableWidgetItem()
                item3.setData(QtCore.Qt.EditRole, int(min(values)))
                item4 = QtWidgets.QTableWidgetItem()
                item4.setData(QtCore.Qt.EditRole, float(np.round(sum(values) / len(values), 4)))
                item5 = QtWidgets.QTableWidgetItem()
                item5.setData(QtCore.Qt.EditRole, int(max(values)))
                item6 = QtWidgets.QTableWidgetItem()
                item6.setData(QtCore.Qt.EditRole, self.txtBackground.toPlainText())

                self.tblAvgStat.setItem(row, 0, item1)
                self.tblAvgStat.setItem(row, 1, item2)
                self.tblAvgStat.setItem(row, 2, item3)
                self.tblAvgStat.setItem(row, 3, item4)
                self.tblAvgStat.setItem(row, 4, item5)
                self.tblAvgStat.setItem(row, 5, item6)
            else:
                item1 = QtWidgets.QTableWidgetItem()
                item1.setData(QtCore.Qt.EditRole, row)
                item2 = QtWidgets.QTableWidgetItem()
                item2.setData(QtCore.Qt.EditRole, x)
                item3 = QtWidgets.QTableWidgetItem()
                item3.setData(QtCore.Qt.EditRole, 0)
                item4 = QtWidgets.QTableWidgetItem()
                item4.setData(QtCore.Qt.EditRole, 0)
                item5 = QtWidgets.QTableWidgetItem()
                item5.setData(QtCore.Qt.EditRole, 0)
                item6 = QtWidgets.QTableWidgetItem()
                item6.setData(QtCore.Qt.EditRole, self.txtBackground.toPlainText())

                self.tblAvgStat.setItem(row, 0, item1)
                self.tblAvgStat.setItem(row, 1, item2)
                self.tblAvgStat.setItem(row, 2, item3)
                self.tblAvgStat.setItem(row, 3, item4)
                self.tblAvgStat.setItem(row, 4, item5)
                self.tblAvgStat.setItem(row, 5, item6)
        else:
            item1 = QtWidgets.QTableWidgetItem()
            item1.setData(QtCore.Qt.EditRole, row)
            item2 = QtWidgets.QTableWidgetItem()
            item2.setData(QtCore.Qt.EditRole, x)
            item3 = QtWidgets.QTableWidgetItem()
            item3.setData(QtCore.Qt.EditRole, 0)
            item4 = QtWidgets.QTableWidgetItem()
            item4.setData(QtCore.Qt.EditRole, 0)
            item5 = QtWidgets.QTableWidgetItem()
            item5.setData(QtCore.Qt.EditRole, 0)
            item6 = QtWidgets.QTableWidgetItem()
            item6.setData(QtCore.Qt.EditRole, self.txtBackground.toPlainText())

            self.tblAvgStat.setItem(row, 0, item1)
            self.tblAvgStat.setItem(row, 1, item2)
            self.tblAvgStat.setItem(row, 2, item3)
            self.tblAvgStat.setItem(row, 3, item4)
            self.tblAvgStat.setItem(row, 4, item5)
            self.tblAvgStat.setItem(row, 5, item6)

    def __refreshPixelStat__(self, roi, index):
        values = roi.getPixelsAt(self.__getScbVal__())
        self.tblROIStatistic.setSortingEnabled(False)
        for x in range(0, len(values)):
            row = self.tblROIStatistic.rowCount()
            self.tblROIStatistic.insertRow(row)
            if values[x] != 0:
                contrast = np.round(int(self.txtBackground.toPlainText()) / values[x], 3)
                delta = abs(values[x] - int(self.txtBackground.toPlainText()))
            else:
                contrast = 1
                delta = abs(values[x] - int(self.txtBackground.toPlainText()))

            item1 = QtWidgets.QTableWidgetItem()
            item1.setData(QtCore.Qt.EditRole, x)
            item2 = QtWidgets.QTableWidgetItem()
            item2.setData(QtCore.Qt.EditRole, index)
            item3 = QtWidgets.QTableWidgetItem()
            item3.setData(QtCore.Qt.EditRole, int(values[x]))
            item4 = QtWidgets.QTableWidgetItem()
            item4.setData(QtCore.Qt.EditRole, float(contrast))
            item5 = QtWidgets.QTableWidgetItem()
            item5.setData(QtCore.Qt.EditRole, int(delta))

            self.tblROIStatistic.setItem(row, 0, item1)
            self.tblROIStatistic.setItem(row, 1, item2)
            self.tblROIStatistic.setItem(row, 2, item3)
            self.tblROIStatistic.setItem(row, 3, item4)
            self.tblROIStatistic.setItem(row, 4, item5)
        self.tblROIStatistic.setSortingEnabled(True)

    def _exportStatClicked_(self):
        self._resetErrorMessage_()
        if self.data is not None:
            rowNr = self.tblAvgStat.currentRow()
            if rowNr >= 0:
                roirow = int(self.tblAvgStat.item(rowNr, 0).text())
                values = self.scene.roi[roirow].getPixels()
                filename = QtWidgets.QFileDialog.getOpenFileName(None, "Select File", "", "Files(*.csv *.txt)")
                if filename[0] != "":
                    with open(filename[0], "w", newline='') as f:
                        writer = csv.writer(f, delimiter=';')
                        for i in range(0, len(self.data)):
                            l = "Layer " + str(i+1)
                            avg = np.round(sum(values[i]) / len(values[i]), 3)
                            strAvg = (str(avg).replace('.', ','))
                            writer.writerow([l])
                            writer.writerow(["ROI", "Min. Intensity", "Avg. Intensity", "Max. Intensity", "global Bg"])
                            writer.writerow([roirow, min(values[i]), strAvg, max(values[i]),
                                             self.txtBackground.toPlainText()])
                            writer.writerow(["Single Pixels:"])
                            writer.writerow(["ID", "Intensity", "Contrast", "Delta"])
                            for x in range(0, len(values[i])):
                                if values[i][x] != 0:
                                    contrast = np.round(int(self.txtBackground.toPlainText()) / values[i][x], 3)
                                    strContrast = str(contrast).replace('.', ',')
                                    delta = abs(values[i][x] - int(self.txtBackground.toPlainText()))
                                    writer.writerow([x, values[i][x], strContrast, delta])
                                else:
                                    writer.writerow([x, values[i][x], 100, self.txtBackground.toPlainText()])

                else:
                    self.lblErrors.setText("Could not export Statistics, because of an invalid filename.")
            else:
                self.lblErrors.setText("Could not export Statistics! Please be sure to select the \"Statistics\" Tab " +
                                       "and an element from the \"ROI Statistics\" table.")
        else:
            self.lblErrors.setText("No data available! Please load in an image.")

    def _cbChange_(self):
        self.scene.add = False
        self.scene.rect = (str(self.cbROI.currentText()) == "Rectangle")
        self.scene.clearCoord()
        self.scene.deletePoints()

    def _detectROIClicked_(self):
        self._resetErrorMessage_()
        self.setStatusWorking()
        QtGui.QGuiApplication.processEvents()
        if self.data is not None:
            self._detectROI_()
        else:
            self.lblErrors.setText("No data available! Please load in an image.")
        self.setStatusIdle()

    def _detectROI_(self):
        if not self.useThresh:
            img = qi2np.rgb_view(self.__qim__[self.__getScbVal__()])
        else:
            img = qi2np.rgb_view(self.scene.getPixThreshAt(self.__getScbVal__()).toImage())

        gray = cv.cvtColor(img, cv.COLOR_BGR2GRAY)
        threshLimit = self.getThreshLimit()
        minW = self.getMinW()
        minH = self.getMinH()
        ret, thresh = cv.threshold(gray, threshLimit, 255, 0)
        retrMode = cv.RETR_TREE
        apprMode = cv.CHAIN_APPROX_SIMPLE
        contours, hierarchy = cv.findContours(thresh, retrMode, apprMode)
        Parallel(n_jobs=self.num_cores, prefer="threads")(
            delayed(self.__doDetection__)(c, minW, minH) for c in contours)

        self._redrawROI_()
        self._refreshStats_()

    def getThreshLimit(self):
        return self.ibThreshDetect.value()

    def getMinW(self):
        return self.ibMinWidth.value()

    def getMinH(self):
        return self.ibMinHeight.value()

    def _setBackgroundClicked_(self):
        self._resetErrorMessage_()
        if self.data is not None:
            if self.twViewStat.currentIndex() == 1:
                self.lblErrors.setText("Can not set Background in this Tab! Please be sure to select the \"View\" Tab.")
            self.scene.setbackground = True
            self.scene.add = False
            self.scene.coord.clear()
        else:
            self.lblErrors.setText("No data available! Please load in an image.")

    # helpers------------------------------------------------------------------
    def _resetErrorMessage_(self):
        self.lblErrors.setText("")

    def _setBackground_(self):
        if len(self.scene.backgroundCoord) == 4:
            values = []
            coords1 = np.around(
                self.__getOriginalCoord__([self.scene.backgroundCoord[0], self.scene.backgroundCoord[1]]))
            coords2 = np.around(
                self.__getOriginalCoord__([self.scene.backgroundCoord[2], self.scene.backgroundCoord[3]]))
            coords1 = self.convertToInt(coords1)
            coords2 = self.convertToInt(coords2)
            if coords1[0] < coords2[0] and coords1[1] < coords2[1]:
                for x in range(coords1[0], coords2[0]):
                    for y in range(coords1[1], coords2[1]):
                        values.append(self.data[self.__getScbVal__()][y][x])
            elif coords1[0] > coords2[0] and coords1[1] < coords2[1]:
                for x in range(coords2[0], coords1[0]):
                    for y in range(coords1[1], coords2[1]):
                        values.append(self.data[self.__getScbVal__()][y][x])
            elif coords1[0] < coords2[0] and coords1[1] > coords2[1]:
                for x in range(coords1[0], coords2[0]):
                    for y in range(coords2[1], coords1[1]):
                        values.append(self.data[self.__getScbVal__()][y][x])
            elif coords1[0] > coords2[0] and coords1[1] > coords2[1]:
                for x in range(coords2[0], coords1[0]):
                    for y in range(coords2[1], coords1[1]):
                        values.append(self.data[self.__getScbVal__()][y][x])
            else:
                self.scene.setBackground = False
                self.scene.backgroundCoord.clear()
                self.scene.deletePoints()
                return
            self.txtBackground.setText(str(int(sum(values) / len(values))))
            self.scene.setBackground = False
            self.scene.backgroundCoord.clear()
            self.scene.deletePoints()
            self._refreshStats_()

    def convertToInt(self, coords):
        c = []
        for coord in coords:
            c.append(int(coord))
        return c

    def __trim__(self, im):
        bg = Image.new(im.mode, im.size, im.getpixel((0, 0)))
        diff = ImageChops.difference(im, bg)
        diff = ImageChops.add(diff, diff, 2.0, -100)
        bbox = diff.getbbox()
        if bbox:
            return im.crop(bbox)

    def __clearTable__(self, tbl):
        for row in reversed(range(0, tbl.rowCount())):
            tbl.removeRow(row)
        return tbl

    def __getOriginalCoord__(self, coord):
        coord[0] = np.round(coord[0] * (1 / self.ratio), 1)
        coord[1] = np.round(coord[1] * (1 / self.ratio), 1)
        return coord

    def __getRectValues__(self, coords1, coords2, roi):
        roi._pixels_ = list()
        for i in range(0, len(self.data)):
            v = list()
            for x in range(int(np.round(coords1[0])), int(np.round(coords2[0]))):
                for y in range(int(np.round(coords1[1])), int(np.round(coords2[1]))):
                    v.append(self.data[i][y][x])
            roi.appendPixelArray(v)
        roi.setupdateNeeded(False)

    def __getPolyValues__(self, singleRoi):
        singleRoi.originalPixelCoord.clear()
        rect = singleRoi.getOriginalPoly().polygon().boundingRect()
        singleRoi._pixels_ = list()
        for i in range(0, len(self.data)):
            v = []
            oriV = []
            xs = [rect.getCoords()[0], rect.getCoords()[2]]
            ys = [rect.getCoords()[1], rect.getCoords()[3]]
            for x in range(int(xs[0]), int(xs[1])):
                for y in range(int(ys[0]), int(ys[1])):
                    if singleRoi.getOriginalPoly().contains(QtCore.QPoint(x, y)) and \
                            QtCore.QPoint(x, y) not in singleRoi.getAllPixelCoord():
                        # c = self.__getOriginalCoord__([x, y])
                        c = [x, y]
                        oriV.append(QtCore.QPoint(c[0], c[1]))
                        v.append(self.data[i][int(c[1])][int(c[0])])
            singleRoi.appendPixelCoordArray(oriV)
            singleRoi.appendPixelArray(v)
        if singleRoi.getArea() == 0:
            singleRoi.setArea(len(singleRoi.getPixels()))
        singleRoi.setupdateNeeded(False)

    def __doPolyAddY_(self, scenePoly, poly, offset, i):
        p = scenePoly.at(i)
        poly.append(QtCore.QPoint(p.x(), p.y() + offset))

    def __doPolyAddX_(self, scenePoly, poly, offset, i):
        p = scenePoly.at(i)
        poly.append(QtCore.QPoint(p.x() + offset, p.y()))

    def __doDetection__(self, c, minWidth, minHeight):
        M = cv.moments(c)
        if M["m00"] != 0:
            coords = []
            for entry in c:
                coords.append(entry[0])
            self.scene.addPoly(coords, minWidth, minHeight)

    def __getScbVal__(self):
        return self.hSbImages.value()-1

    def __getGrayValue__(self, im, x, y):
        p = im.getpixel((x, y))
        newp = int(0.30 * p[0] + 0.59 * p[1] + 0.11 * p[2])
        return (newp, newp, newp)

    # unused-----------------------------------------------------------------
    def __countPixel__(self, coords1, coords2):
        pixels = 0
        for y in range(coords1[0], coords2[0]):
            for x in range(coords1[1], coords2[1]):
                if self.data[self.__getScbVal__()][y][x] > -1:
                    pixels = pixels + 1
        return pixels


class StackDataGUI(Ui_Dialog):

    def __init__(self, dialog):
        Ui_Dialog.__init__(self)
        self.sceneXY = GraphicsScene.GraphicsSceneV()
        self.sceneYZ = GraphicsScene.GraphicsSceneV()
        self.sceneXZ = GraphicsScene.GraphicsSceneV()

    def initialize(self):
        self.sbXY.valueChanged.connect(self.resizeXY)
        self.sbYZ.valueChanged.connect(self.resizeYZ)
        self.sbXZ.valueChanged.connect(self.resizeXZ)

    def resizeXY(self):
        scale = self.sbXY.value()
        w = self.sceneXY.getOrigWidth()
        h = self.sceneXY.getOrigHeight()
        newcp = []
        newgp = []
        for imagenr in range(0, self.stack_size_sz):
            cp = self.sceneXY.getPixColorAt(imagenr)
            gp = self.sceneXY.getPixGrayAt(imagenr)
            newcp.append(cp.scaled(w * scale, h * scale, QtCore.Qt.KeepAspectRatio))
            newgp.append(gp.scaled(w * scale, h * scale, QtCore.Qt.KeepAspectRatio))
        self.sceneXY.setPixColor(newcp)
        self.sceneXY.setPixGray(newgp)
        self.sceneXY.setSceneRect(QtCore.QRectF(0, 0, w * scale, h * scale))
        self.gvViewXY.setSceneRect(self.sceneXY.sceneRect())
        self.__setXYGray__()

    def resizeYZ(self):
        scale = self.sbYZ.value()
        w = self.sceneYZ.getOrigWidth()
        h = self.sceneYZ.getOrigHeight()
        newcp = []
        newgp = []
        for imagenr in range(0, self.stack_size_sx):
            cp = self.sceneYZ.getPixColorAt(imagenr)
            gp = self.sceneYZ.getPixGrayAt(imagenr)
            newcp.append(cp.scaled(w * scale, h * scale, QtCore.Qt.KeepAspectRatio))
            newgp.append(gp.scaled(w * scale, h * scale, QtCore.Qt.KeepAspectRatio))
        self.sceneYZ.setPixColor(newcp)
        self.sceneYZ.setPixGray(newgp)
        self.sceneYZ.setSceneRect(QtCore.QRectF(0, 0, w * scale, h * scale))
        self.gvViewYZ.setSceneRect(self.sceneYZ.sceneRect())
        self.__setYZGray__()
        
    def resizeXZ(self):
        scale = self.sbXZ.value()
        w = self.sceneXZ.getOrigWidth()
        h = self.sceneXZ.getOrigHeight()
        newcp = []
        newgp = []
        for imagenr in range(0, self.stack_size_sy):
            cp = self.sceneXZ.getPixColorAt(imagenr)
            gp = self.sceneXZ.getPixGrayAt(imagenr)
            newcp.append(cp.scaled(w * scale, h * scale, QtCore.Qt.KeepAspectRatio))
            newgp.append(gp.scaled(w * scale, h * scale, QtCore.Qt.KeepAspectRatio))
        self.sceneXZ.setPixColor(newcp)
        self.sceneXZ.setPixGray(newgp)
        self.sceneXZ.setSceneRect(QtCore.QRectF(0, 0, w * scale, h * scale))
        self.gvViewXZ.setSceneRect(self.sceneXZ.sceneRect())
        self.__setXZGray__()

    def __setXYGray__(self):
        if self.cbGrayXY.isChecked():
            self.sceneXY.items()[0].setPixmap(self.sceneXY.getPixGrayAt(self.hSbImagesXY.value()-1))
        else:
            self.sceneXY.items()[0].setPixmap(self.sceneXY.getPixColorAt(self.hSbImagesXY.value()-1))

    def __setXZGray__(self):
        if self.cbGrayXZ.isChecked():
            self.sceneXZ.items()[0].setPixmap(self.sceneXZ.getPixGrayAt(self.hSbImagesXZ.value()-1))
        else:
            self.sceneXZ.items()[0].setPixmap(self.sceneXZ.getPixColorAt(self.hSbImagesXZ.value()-1))

    def __setYZGray__(self):
        if self.cbGrayYZ.isChecked():
            self.sceneYZ.items()[0].setPixmap(self.sceneYZ.getPixGrayAt(self.hSbImagesYZ.value()-1))
        else:
            self.sceneYZ.items()[0].setPixmap(self.sceneYZ.getPixColorAt(self.hSbImagesYZ.value()-1))

    def readData(self, stack):
        self.gvViewXZ.setStyleSheet("background: transparent")
        self.gvViewXY.setStyleSheet("background: transparent")
        self.gvViewYZ.setStyleSheet("background: transparent")
        self.stack_size_sz = np.size(stack, 0)
        self.stack_size_sy = np.size(stack, 1)
        self.stack_size_sx = np.size(stack, 2)

        self.fillMatrix(stack)

        plt.close('all')
        self.sceneXY.pixColorXY = []
        self.sceneXY.pixGrayXY = []
        for k in range(0, self.stack_size_sz):
            s = self.matrix[k, :, :]

            fig = plt.figure(frameon=False)
            ax = plt.Axes(fig, [0., 0., 1., 1.])

            ax.set_axis_off()
            fig.add_axes(ax)
            ax.imshow(s)
            buf = io.BytesIO()

            plt.savefig(buf, format='png', bbox_inches='tight', pad_inches=0)
            plt.close(fig)
            buf.seek(0)
            im = Image.open(buf)
            #im = self.__trim__(im)

            # convert this image and save it into a qpixmap so it can be visualized in graphics view
            d = im.tobytes("raw", "RGBA")
            qimXY = QtGui.QImage(d, im.size[0], im.size[1], QtGui.QImage.Format_RGBA8888)
            pixColorXY = QtGui.QPixmap.fromImage(qimXY)
            # gray version
            [[im.putpixel((x, y), self.__getGrayValue__(im, x, y)) for y in range(0,im.height)] for x in range(0, im.width)]
            d = im.tobytes("raw", "RGBA")
            qimXY = QtGui.QImage(d, im.size[0], im.size[0], QtGui.QImage.Format_RGBA8888)
            pixGrayXY = QtGui.QPixmap.fromImage(qimXY)

            self.sceneXY.addPixColorAt(pixColorXY.scaled(s.shape[0], s.shape[0]), 0)
            self.sceneXY.addPixGrayAt(
                pixGrayXY.scaled(s.shape[0], s.shape[0], QtCore.Qt.KeepAspectRatio), 0)
            buf.close()

        plt.close('all')
        self.sceneYZ.pixColor = []
        self.sceneYZ.pixGray = []

        for k in range(0, self.stack_size_sx):
            t = self.matrix[:, :, k]
            t = np.swapaxes(t, 0, 1)
            plt.imshow(t)
            fig = plt.figure(frameon=False)
            ax = plt.Axes(fig, [0., 0., 1., 1.])

            ax.set_axis_off()
            fig.add_axes(ax)
            ax.imshow(t)
            buf = io.BytesIO()

            plt.savefig(buf, format='png', bbox_inches='tight', pad_inches=0)
            plt.close(fig)
            buf.seek(0)
            im = Image.open(buf)
            #im = self.__trim__(im)
            #img = im.convert("L")

            d = im.tobytes("raw", "RGBA")
            qimYZ = QtGui.QImage(d, im.size[0], im.size[1], QtGui.QImage.Format_RGBA8888)
            pixColorYZ = QtGui.QPixmap.fromImage(qimYZ)
            [[im.putpixel((x, y), self.__getGrayValue__(im, x, y)) for y in range(0,im.height)] for x in range(0, im.width)]
            d = im.tobytes("raw", "RGBA")
            qimYZ = QtGui.QImage(d, im.size[0], im.size[1], QtGui.QImage.Format_RGBA8888)
            pixGrayYZ = QtGui.QPixmap.fromImage(qimYZ)
            s = t.shape
            self.sceneYZ.addPixColorAt(pixColorYZ.scaled(s[1], s[0], QtCore.Qt.KeepAspectRatio), 0)
            self.sceneYZ.addPixGrayAt(
                 pixGrayYZ.scaled(s[1], s[0], QtCore.Qt.KeepAspectRatio), 0)#.scaled(pixColorYZ.width(), pixColorYZ.height(), QtCore.Qt.KeepAspectRatio), 0)
            buf.close()

        plt.close('all')
        self.sceneXZ.pixColor = []
        self.sceneXZ.pixGray = []

        for k in range(0, self.stack_size_sy):
            m = self.matrix[:, k, :]
            plt.imshow(m, cmap='Greys', interpolation='nearest')
            fig = plt.figure(frameon=False)
            ax = plt.Axes(fig, [0., 0., 1., 1.])

            ax.set_axis_off()
            fig.add_axes(ax)
            ax.imshow(m)
            buf = io.BytesIO()

            plt.savefig(buf, format='png', bbox_inches='tight', pad_inches=0)
            plt.close(fig)
            buf.seek(0)
            im = Image.open(buf)
            #im = self.__trim__(im)

            d = im.tobytes("raw", "RGBA")
            qimXZ = QtGui.QImage(d, im.size[0], im.size[1], QtGui.QImage.Format_RGBA8888)
            pixColorXZ = QtGui.QPixmap.fromImage(qimXZ)

            [[im.putpixel((x, y), self.__getGrayValue__(im, x, y)) for y in range(0,im.height)] for x in range(0, im.width)]
            d = im.tobytes("raw", "RGBA")
            qimXZ = QtGui.QImage(d, im.size[0], im.size[1], QtGui.QImage.Format_RGBA8888)
            pixGrayXZ = QtGui.QPixmap.fromImage(qimXZ)
            s = m.shape
            self.sceneXZ.addPixColorAt(pixColorXZ.scaled(s[1], s[0], QtCore.Qt.KeepAspectRatio), 0)
            self.sceneXZ.addPixGrayAt(
                 pixGrayXZ.scaled(s[1], s[0], QtCore.Qt.KeepAspectRatio), 0)#.scaled(pixColorXZ.width(), pixColorXZ.height(), QtCore.Qt.KeepAspectRatio), 0)
            buf.close()

        self.sceneXY.addPixmap(pixColorXY)
        self.sceneYZ.addPixmap(pixColorYZ)
        self.sceneXZ.addPixmap(pixColorXZ)

        self.gvViewXY.setScene(self.sceneXY)
        self.gvViewYZ.setScene(self.sceneYZ)
        self.gvViewXZ.setScene(self.sceneXZ)

        self.gvViewXY.show()
        self.gvViewYZ.show()
        self.gvViewXZ.show()

        self.lblImageNrXY.setText(str(1) + "\\" + str(self.stack_size_sz))
        self.lblImageNrYZ.setText(str(1) + "\\" + str(self.stack_size_sx))
        self.lblImageNrXZ.setText(str(1) + "\\" + str(self.stack_size_sx))

        self.hSbImagesXY.setValue(1)
        self.hSbImagesYZ.setValue(1)
        self.hSbImagesXZ.setValue(1)

        self.hSbImagesXY.valueChanged.connect(self.__imgScaleBarXYChanged__)
        self.hSbImagesYZ.valueChanged.connect(self.__imgScaleBarYZChanged__)
        self.hSbImagesXZ.valueChanged.connect(self.__imgScaleBarXZChanged__)

        self.hSbImagesXY.setMaximum(self.stack_size_sz)
        self.hSbImagesYZ.setMaximum(self.stack_size_sx)
        self.hSbImagesXZ.setMaximum(self.stack_size_sy)

        self.cbGrayXY.stateChanged.connect(self.__setXYGray__)
        self.cbGrayXZ.stateChanged.connect(self.__setXZGray__)
        self.cbGrayYZ.stateChanged.connect(self.__setYZGray__)

        self.sceneXY.setOrigWidth(self.sceneXY.getPixColorAt(0).width())
        self.sceneXZ.setOrigWidth(self.sceneXZ.getPixColorAt(0).width())
        self.sceneYZ.setOrigWidth(self.sceneYZ.getPixColorAt(0).width())

        self.sceneXY.setOrigHeight(self.sceneXY.getPixColorAt(0).height())
        self.sceneXZ.setOrigHeight(self.sceneXZ.getPixColorAt(0).height())
        self.sceneYZ.setOrigHeight(self.sceneYZ.getPixColorAt(0).height())


        self.__setXYGray__()
        self.__setXZGray__()
        self.__setYZGray__()

    def fillMatrix(self, stack):
        self.matrix = np.zeros((self.stack_size_sz, self.stack_size_sy, self.stack_size_sx))
        for k in range(0, self.stack_size_sz):
            self.matrix[k, :, :] = stack[k]

    def __trim__(self, im):
        bg = Image.new(im.mode, im.size, im.getpixel((0, 0)))
        diff = ImageChops.difference(im, bg)
        diff = ImageChops.add(diff, diff, 2.0, -100)
        bbox = diff.getbbox()
        if bbox:
            return im.crop(bbox)

    def __imgScaleBarXYChanged__(self):
        i = self.hSbImagesXY.value() - 1
        self.lblImageNrXY.setText(str(i + 1) + "\\" + str(self.stack_size_sz))
        if self.cbGrayXY.isChecked():
            self.sceneXY.items()[0].setPixmap(
                self.sceneXY.getPixGrayAt(i))
        else:
            self.sceneXY.items()[0].setPixmap(
                self.sceneXY.getPixColorAt(i))

    def __imgScaleBarYZChanged__(self):
        i = self.hSbImagesYZ.value() - 1
        self.lblImageNrYZ.setText(str(i + 1) + "\\" + str(self.stack_size_sx))
        if self.cbGrayYZ.isChecked():
            self.sceneYZ.items()[0].setPixmap(
                self.sceneYZ.getPixGrayAt(i))
        else:
            self.sceneYZ.items()[0].setPixmap(
                self.sceneYZ.getPixColorAt(i))

    def __imgScaleBarXZChanged__(self):
        i = self.hSbImagesXZ.value() - 1
        self.lblImageNrXZ.setText(str(i + 1) + "\\" + str(self.stack_size_sx))
        if self.cbGrayXZ.isChecked():
            self.sceneXZ.items()[0].setPixmap(
                self.sceneXZ.getPixGrayAt(i))
        else:
            self.sceneXZ.items()[0].setPixmap(
                self.sceneXZ.getPixColorAt(i))

    def __getGrayValue__(self, im, x, y):
        p = im.getpixel((x,y))
        newp = int(0.30*p[0] + 0.59*p[1] + 0.11*p[2])
        return (newp, newp, newp)


# main ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#


if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = PySpot(MainWindow)
    MainWindow.show()
    MainWindow.setWindowTitle("PySpot")
    sys.exit(app.exec_())

