# -*- coding: utf-8 -*-
"""
Created on Mon Mar 25

@author: hannah janout
"""
from PyQt5 import QtCore, QtWidgets, QtGui
from enum import Enum
import sys


class ROIType(Enum):
    Rectangle = 0
    Polygon = 1


# class for saving ROI and their coordinates, height, width and area
class ROI:
    width = 0
    height = 0
    x = list()
    y = list()
    area = 0
    updateNeeded = True
    RoiType = ROIType.Rectangle
    _pixels_ = list()
    originalCoord = list()

    def __init__(self):
        self._pixels_ = list(self._pixels_)

# getters++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def getX(self):
        return self.x

    def getY(self):
        return self.y

    def getArea(self):
        return self.area

    def getHeight(self):
        return self.height

    def getWidth(self):
        return self.width

    def getType(self):
        return self.RoiType

    def getPixels(self):
        return self._pixels_

    def getPixelsAt(self, i):
        return self._pixels_[i]

    def getUpdated(self):
        return self.updateNeeded

    def getOriginalCoord(self):
        return self.originalCoord

# setters++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def setOriginalCoord(self, c):
        self.originalCoord = c

    def setWidth(self, width):
        self.width = width

    def setHeight(self, height):
        self.height = height

    def setPixel(self, p):
        self._pixels_ = p

    def setPixelAt(self, p, i):
        self._pixels_[i] = p

    def appendPixelArray(self,p):
        self._pixels_.append(p)

    def setArea(self, area):
        self.area = area

    def setupdateNeeded(self, state):
        self.updateNeeded = state

# helpers++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def clearPixel(self):
        self._pixels_.clear()

    def appendPixel(self, p):
        self._pixels_.append(p)

    def shiftX(self, offset):
        self.updateNeeded = True
        self.x[0] = self.x[0] + offset
        self.x[1] = self.x[1] + offset
        newCoord = []
        for c in self.originalCoord:
            newCoord.append(QtCore.QPoint(c.x() + offset, c.y()))
        self.originalCoord = newCoord

    def shiftY(self, offset):
        self.updateNeeded = True
        self.y[0] = self.y[0] + offset
        self.y[1] = self.y[1] + offset
        newCoord = []
        for c in self.originalCoord:
            newCoord.append(QtCore.QPoint(c.x(), c.y() + offset))
        self.originalCoord = newCoord

class RectROI(ROI):
    rect = QtWidgets.QGraphicsRectItem()
    
    def __init__(self, item):        
        self.rect = item
        self.x = [self.rect.rect().topLeft().x(),  self.rect.rect().bottomRight().x()]
        self.y = [self.rect.rect().topLeft().y(),  self.rect.rect().bottomRight().y()]
        self.width = abs(self.x[0] - self.x[1])
        self.height = abs(self.y[0] - self.y[1])
        self.area = self.width * self.height
        self.RoiType = ROIType.Rectangle
        self.originalCoord = list(self.originalCoord)

# getters++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def getRect(self):
        return self.rect

    def getTopLeft(self):
        return self.rect.rect().topLeft()

    def getBottomRight(self):
        return self.rect.rect().bottomRight()

    def getSceneWidth(self):
        return int(self.rect.rect().width())

    def getSceneHeight(self):
        return int(self.rect.rect().height())

# setters++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def setX(self, x):
        self.x = x
        
    def setY(self, y):
        self.y = y
        
    def setRect(self, rect):
        self.rect.setRect(rect)

# helpers++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def adjustAfterSet(self):
        self.width = abs(self.x[0] - self.x[1])
        self.height = abs(self.y[0] - self.y[1])
        self.area = self.width * self.height

    def adjustAfterSetOriginal(self):
        self.width = abs(self.originalCoord[0].x() - self.originalCoord[1].x())
        self.height = abs(self.originalCoord[0].y() - self.originalCoord[1].y())
        self.area = self.width * self.height
        self.x = [self.getOriginalCoord()[0].x(), self.getOriginalCoord()[1].x()]
        self.y = [self.getOriginalCoord()[0].y(), self.getOriginalCoord()[1].y()]

class PolyROI(ROI):
    
    poly = QtWidgets.QGraphicsPolygonItem()
    originalPixelCoord = []
    originalX = []
    originalY = []
    originalPoly = QtWidgets.QGraphicsPolygonItem()

    def __init__(self, item, scale):
        self.poly = item
        self.RoiType = ROIType.Polygon
        self.x.clear()
        self.y.clear()
        self.originalCoord = list(self.originalCoord)
        self.originalPixelCoord = list(self.originalPixelCoord)
        self.originalX = [sys.maxsize, 0]
        self.originalY = [sys.maxsize, 0]
        self.initializeOriginalCoord(scale)
        self.initializeOriginalXY()
        self.initializeOriginalPoly()
        self.initializeHeight()
        self.initializeWidth()
        self.initializeArea()
        self.x = self._calculateOriX_()
        self.y = self._calculateOriY_()

# initialize++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def initializeOriginalCoord(self, scale):
        for i in range(0, self.getPoly().polygon().count()):
            p = self.getPoly().polygon().at(i)
            self.appendX(p.x() * (1 / scale))
            self.appendY(p.y() * (1 / scale))
            self.originalCoordappend(QtCore.QPoint(p.x() * (1 / scale), p.y() * (1 / scale)))

    def initializeOriginalXY(self):
        for p in self.originalCoord:
            if p.x() < self.originalX[0]:
                self.originalX[0] = p.x()
            elif p.x() > self.originalX[1]:
                self.originalX[1] = p.x()
            if p.y() < self.originalY[0]:
                self.originalY[0] = p.y()
            elif p.y() > self.originalY[1]:
                self.originalY[1] = p.y()
        self.setX(self.originalX)
        self.setY(self.originalY)

    def initializeOriginalPoly(self):
        poly = QtGui.QPolygonF()
        for p in self.originalCoord:
            poly.append(p)
        self.originalPoly = QtWidgets.QGraphicsPolygonItem(poly)

    def initializeHeight(self):
        self.height = self._calculateHeight_()
     
    def initializeWidth(self):
        self.width = self._calculateWidth_()
        
    def initializeX(self):
        self.x = self._calculateX_()
        
    def initializeY(self):
        self.y = self._calculateY_()
        
    def initializeArea(self):
        self.area = 0
        rect = self.getOriginalPoly().polygon().boundingRect()
        coords = []
        xs = [rect.getCoords()[0], rect.getCoords()[2]]
        ys = [rect.getCoords()[1], rect.getCoords()[3]]
        for x in range(int(xs[0]), int(xs[1])):
            for y in range(int(ys[0]), int(ys[1])):
                if self.originalPoly.contains(QtCore.QPoint(x, y)):
                    self.area = self.area + 1
                    coords.append(QtCore.QPoint(x, y))

# calculations++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def _calculateX_(self):
        return [min(self.x), max(self.x)]
    
    def _calculateY_(self):
        return [min(self.y), max(self.y)]

    def _calculateOriX_(self):
        return [min(self.originalX), max(self.originalX)]

    def _calculateOriY_(self):
        return [min(self.originalY), max(self.originalY)]
            
    def _calculateHeight_(self):
        return abs(max(self.y) - min(self.y))
    
    def _calculateWidth_(self):
        return abs(max(self.x) - min(self.x))

# setters++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def setX(self, x):
        self.x = x
        
    def setY(self, y):
        self.y = y
        
    def setPixelCoord(self, pixel):
        self.originalPixelCoord = pixel

    def setAfterCopy(self, scale):
        self.originalCoord.clear()
        for i in range(0, self.getPoly().polygon().count()):
            p = self.getPoly().polygon().at(i)
            self.appendX(p.x() * (1 / scale))
            self.appendY(p.y() * (1 / scale))
            self.originalCoord.append(QtCore.QPoint(p.x() * (1 / scale), p.y() * (1 / scale)))
        self.initializeOriginalXY()
        self.initializeHeight()
        self.initializeWidth()
        self.initializeOriginalPoly()
        self.initializeArea()
        self.x = self._calculateOriX_()
        self.y = self._calculateOriY_()

# getters++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def getPixelCoord(self, i):
        return self.originalPixelCoord[i]

    def getAllPixelCoord(self):
        return self.originalPixelCoord

    def getPoly(self):
        return self.poly

    def getOriginalPoly(self):
        return self.originalPoly

    def getOriginalX(self):
        return self.originalX

    def getOriginalY(self):
        return self.originalY
# appends+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def appendX(self, x):
        self.x.append(x)
        
    def appendY(self, y):
        self.y.append(y)

    def appendPixelCoord(self, pixel):
        self.originalPixelCoord.append(pixel)

    def appendPixelCoordArray(self, pixel):
        self.originalPixelCoord.append(pixel)

    def originalCoordappend(self, p):
        self.originalCoord.append(p)

# helpers+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def adjustAfterSet(self):
        self.x = self._calculateX_()
        self.y = self._calculateY_()
        self.width = self._calculateWidth_()
        self.height = self._calculateHeight_()

    def shiftX(self, offset):
        self.setupdateNeeded(True)
        ROI.shiftX(self, offset)
        self.initializeOriginalPoly()

    def shiftY(self, offset):
        self.setupdateNeeded(True)
        ROI.shiftY(self, offset)
        self.initializeOriginalPoly()